@@ -584,8 +584,6 @@ class MainWindow():
                 else:
                     new_name = name + ('.' if ext else '') + ext
 
-                any_changes = (new_name != orig_name) or any_changes
-
                 self.model.set_value(iter, COL_NEW_NAME, new_name)
                 renamed_uri = file_obj.get_pending_uri(new_name)
                 if renamed_uri in self.renamed_uris:
@@ -601,10 +599,15 @@ class MainWindow():
                     self.error_label.set_text(_("'%s' is not writeable.") % file_obj.get_path_or_uri_for_display())
                     self.rename_button.set_sensitive(False)
                 self.renamed_uris.append(renamed_uri)
-                iter = self.model.iter_next(iter)
-                index += 1
+                any_changes = (new_name != orig_name) or any_changes
             except Exception as e:
                 print(e)
+                self.infobar.show()
+                self.error_label.set_text(_("'%s' %s.") % (file_obj.get_path_or_uri_for_display(), str(e)))
+                self.model.set_value(iter, COL_NEW_NAME, orig_name)
+                self.renamed_uris.append(file_obj.uri)
+            iter = self.model.iter_next(iter)
+            index += 1
         self.rename_button.set_sensitive(any_changes)
 
     def replace_text(self, index, string):
@@ -613,27 +616,27 @@ class MainWindow():
         find = self.find_entry.get_text()
         replace = self.replace_entry.get_text()
         replace = self.inject(index, replace)
-        if regex:
-            try:
+        try:
+            if regex:
                 if case:
                     return re.sub(find, replace, string)
                 else:
                     reg = re.compile(find, re.IGNORECASE)
                     return reg.sub(replace, string)
-            except re.error:
-                return string
-        else:
-            find = find.replace("*", "~~~REGSTAR~~~")
-            find = find.replace("?", "~~~REGQUES~~~")
-            find = re.escape(find)
-            find = find.replace(re.escape("~~~REGSTAR~~~"), ".+")
-            find = find.replace(re.escape("~~~REGQUES~~~"), ".")
-            if case:
-                reg = re.compile(find)
-                return reg.sub(replace, string)
             else:
-                reg = re.compile(find, re.IGNORECASE)
-                return reg.sub(replace, string)
+                find = find.replace("*", "~~~REGSTAR~~~")
+                find = find.replace("?", "~~~REGQUES~~~")
+                find = re.escape(find)
+                find = find.replace(re.escape("~~~REGSTAR~~~"), ".+")
+                find = find.replace(re.escape("~~~REGQUES~~~"), ".")
+                if case:
+                    reg = re.compile(find)
+                    return reg.sub(replace, string)
+                else:
+                    reg = re.compile(find, re.IGNORECASE)
+                    return reg.sub(replace, string)
+        except re.error:
+            return string
 
     def remove_text(self, index, string):
         length = len(string) - 1
@@ -651,20 +654,17 @@ class MainWindow():
         text = self.inject(index, text)
         length = len(string) - 1
         from_index = self.insert_spin.get_value_as_int() - 1
-        if from_index >= length:
-            if self.insert_reverse_check.get_active():
-                return text+string
-            else:
-                return string+text
-        else:
-            if self.insert_reverse_check.get_active():
-                from_index = length - from_index + 1
+        a = len(string)
+        b = len(text)
+        if self.insert_reverse_check.get_active():
+            diff = max(0, a - from_index)
             if self.overwrite_check.get_active():
-                if len(text) >= length:
-                    return text
-                else:
-                    catchup = from_index + len(text)
-                    return string[0:from_index] + text + string[catchup:]
+                return string[0:diff] + text + string[diff + b:]
+            else:
+                return string[0:diff] + text + string[diff:]
+        else:
+            if self.overwrite_check.get_active():
+                return string[0:from_index] + text + string[from_index + b:]
             else:
                 return string[0:from_index] + text + string[from_index:]
 
